#include "CombinedDistance.h"

namespace armarx::navigation::human
{
    CombinedDistance::CombinedDistance(const double maxOrientationInfluence,
                                       const double movementInfluence) :
        euclidean(EuclideanDistance()),
        orientation(OrientationDistance()),
        movement(MovementDistance()),
        maxOrientationInfluence(maxOrientationInfluence),
        movementInfluence(movementInfluence)
    {
    }

    double
    CombinedDistance::computeDistance(const Human& h1, const Human& h2) const
    {
        return euclidean.computeDistance(h1, h2)
               // scales the euclidean distance by a factor in [1, maxOrientationInfluence]
               * (1 + (maxOrientationInfluence - 1) * orientation.computeDistance(h1, h2))
               // scales the euclidean distance by a factor in [1, inf) depending on influence
               * (1 + movementInfluence * movement.computeDistance(h1, h2));
    }
} // namespace armarx::navigation::human
