/**
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Timo Weberruß ( timo dot weberruss at student dot kit dot edu )
 * @date       2022
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include "DistanceFunction.h"
#include "EuclideanDistance.h"
#include "MovementDistance.h"
#include "OrientationDistance.h"


namespace armarx::navigation::human
{
    /**
     * @brief The CombinedDistance class combines the standard euclidean distance multiplicatively
     * with an orientation distance in such a way that the distance is always at least the euclidean
     * distance but will be scaled by up to the maximum influence factor if the people face away
     * from each other.
     */
    class CombinedDistance : public DistanceFunction
    {
    public:
        /**
         * @brief CombinedDistance Creates a new combined distance with the specified orientation scale factor
         * @param maxOrientationInfluence the euclidean distance between the humans will be scaled
         * by up to this factor if they face away from each other
         * @param movementInfluence scales the difference in velocity of the humans before applying
         * it to the total distance
         */
        CombinedDistance(double maxOrientationInfluence, double movementInfluence);
        double computeDistance(const Human& h1, const Human& h2) const override;

    private:
        const EuclideanDistance euclidean;
        const OrientationDistance orientation;
        const MovementDistance movement;
        const double maxOrientationInfluence;
        const double movementInfluence;
    };
} // namespace armarx::navigation::human
