/**
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Timo Weberruß ( timo dot weberruss at student dot kit dot edu )
 * @date       2022
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <armarx/navigation/human/types.h>


namespace armarx::navigation::human
{
    /**
     * @brief A distance function is able to compute a distance between two humans.
     * The exact semantics of this distance depends on the implementation. For example,
     * a HumanGrouper uses distances two group humans together (the smaller the distance between
     * two humans, the more likely it is for them to belong to the same social group).
     */
    class DistanceFunction
    {
    public:
        DistanceFunction() = default;
        virtual ~DistanceFunction() = default;

        /**
         * @brief computeDistance computes the (semantic or literal, depending on the implementation) distance between the given humans
         * @param h1 the first human
         * @param h2 the second human
         * @return the distance between the humans (greater values correspond to humans that are further away)
         */
        virtual double computeDistance(const Human& h1, const Human& h2) const = 0;
    };
} // namespace armarx::navigation::human
