/**
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Timo Weberruß ( timo dot weberruss at student dot kit dot edu )
 * @date       2022
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <armarx/navigation/human/types.h>


namespace armarx::navigation::human
{
    /**
     * @brief A strategy encapsuling a way to create a shape describing the special area occupied
     * by a social human group
     */
    class GroupShapeGenerator
    {
    public:
        GroupShapeGenerator() = default;
        virtual ~GroupShapeGenerator() = default;

        /**
         * @brief createShape creates and returns the shape describing the given group
         * @param group a social grouping of at least one human
         * @return a polygon describing the proxemic zone occupied by the given group (not including
         * the zones of the individual members of the group)
         */
        virtual shapes::Polygon createShape(const HumanGroup& group) const = 0;
    };
} // namespace armarx::navigation::human
