#include "HumanGrouper.h"

#include "CombinedDistance.h"
#include "ConvexHullGenerator.h"

namespace armarx::navigation::human
{
    HumanGrouper::HumanGrouper(const GroupingSettings& settings) :
        distanceFunction(std::make_unique<CombinedDistance>(settings.maxOrientationInfluence,
                                                            settings.movementInfluence)),
        shapeGenerator(std::make_unique<ConvexHullGenerator>()),
        settings(settings)
    {
    }

    void
    HumanGrouper::updateHumans(const std::vector<human::Human>& newHumans)
    {
        currentHumans_ = newHumans;
    }

    std::vector<human::HumanGroup>
    HumanGrouper::getCurrentGroups()
    {
        const DateTime now = DateTime::Now();

        std::vector<human::HumanGroup> groups = {};

        // maps index of human in currentHumans to index of group in groups
        std::unordered_map<uint64_t, uint64_t> humanGroupMap;

        for (uint64_t parentIdx = 0; parentIdx < currentHumans_.size(); parentIdx++)
        {
            const Human parent = currentHumans_.at(parentIdx);

            // if no group exists yet for this human, create one just for them
            if (humanGroupMap.count(parentIdx) == 0)
            {
                const human::HumanGroup group = {{{}}, {parent}, now};
                groups.push_back(group);
                humanGroupMap[parentIdx] = groups.size() - 1;
            }

            for (uint64_t childIdx = 0; childIdx < currentHumans_.size(); childIdx++)
            {
                const Human child = currentHumans_.at(childIdx);

                if (this->distanceFunction->computeDistance(parent, child) <
                    settings.groupingThreshold)
                {
                    humanGroupMap[childIdx] = humanGroupMap[parentIdx];
                    groups.at(humanGroupMap[childIdx])
                        .humans.push_back(currentHumans_.at(childIdx));
                }
            }
        }

        for (human::HumanGroup group : groups)
        {
            group.shape = shapeGenerator->createShape(group);
        }

        return groups;
    }

} // namespace armarx::navigation::human
