#include "OrientationDistance.h"

#include <boost/math/constants/constants.hpp>
#include <math.h>

namespace armarx::navigation::human
{
    OrientationDistance::OrientationDistance(const double max, const double min) :
        max(max), min(min)
    {
    }

    double
    OrientationDistance::computeDistance(const Human& h1, const Human& h2) const
    {
        // ranges from 0 to 1
        const double factor = getOrientationFactor(h1, h2);

        return min + factor * (max - min);
    }


    double
    OrientationDistance::getOrientationFactor(const Human& h1, const Human& h2)
    {
        const double lineOrientation = getLineOrientation(h2, h1);
        const double angleH1 = Eigen::Rotation2Dd(h1.pose.linear()).angle();
        const double angleH2 = Eigen::Rotation2Dd(h2.pose.linear()).angle();

        printf("line orientation = %.2f\n", lineOrientation);
        printf("angle h1 = %.2f, angle h2 = %.2f\n", angleH1, angleH2);

        // assuming the angles to be in the interval [0, 2pi]
        const double deviationFirst = std::abs(normOrientation(angleH1 - lineOrientation));
        const double deviationSecond = std::abs(normOrientation(angleH2 - (lineOrientation + 1)));

        printf("deviation first = %.2f, second = %.2f\n", deviationFirst, deviationSecond);

        // ranges from 0 (looking directly at each other) to 1 (back to back)
        return (deviationFirst + deviationSecond) / 2;
    }

    double
    OrientationDistance::getLineOrientation(const Human& h2, const Human& h1)
    {
        const double dx = (h1.pose.translation() - h2.pose.translation()).x();
        const double dy = (h1.pose.translation() - h2.pose.translation()).y();
        const double lineOrientation = atan2(dy, dx);

        return lineOrientation;
    }

    // scales orientation to (-1, 1]
    double
    OrientationDistance::normOrientation(const double orientation)
    {
        double convertedOrientation = orientation / M_PI;
        // brings orientation to [0, 2)
        convertedOrientation = std::fmod(((std::fmod(convertedOrientation, 2)) + 2), 2);
        // brings orientation to [-1, 1)
        convertedOrientation -= static_cast<int>(convertedOrientation > 1) * 2;
        return convertedOrientation;
    }
} // namespace armarx::navigation::human
